@@ -14,11 +14,17 @@ from dask.base import is_dask_collection
 from dask.delayed import Delayed, delayed
 from dask.utils import is_arraylike, is_dataframe_like
 
-from .bins import normalize_bins_range
-from .core import AggHistogram, factory
+from dask_histogram.bins import normalize_bins_range
+from dask_histogram.core import AggHistogram, factory
 
 if TYPE_CHECKING:
-    from .typing import BinArg, BinType, DaskCollection, RangeArg, RangeType
+    from dask_histogram.typing import (
+        BinArg,
+        BinType,
+        DaskCollection,
+        RangeArg,
+        RangeType,
+    )
 
 import dask_histogram
 
@@ -78,7 +84,11 @@ class Histogram(bh.Histogram, family=dask_histogram):
         self._staged: AggHistogram | None = None
 
     def concrete_fill(
-        self, *args: Any, weight: Any | None = None, sample=None, threads=None
+        self,
+        *args: Any,
+        weight: Any | None = None,
+        sample: Any | None = None,
+        threads: int | None = None,
     ) -> Histogram:
         """Fill the histogram with concrete data (not a Dask collection).
 
@@ -111,7 +121,7 @@ class Histogram(bh.Histogram, family=dask_histogram):
             )
         return super().fill(*args, weight=weight, sample=sample, threads=threads)
 
-    def fill(
+    def fill(  # type: ignore
         self,
         *args: DaskCollection,
         weight: DaskCollection | None = None,
@@ -315,7 +325,7 @@ class Histogram(bh.Histogram, family=dask_histogram):
             ret += " # (has staged fills)"
         return ret
 
-    def visualize(self, *args, **kwargs) -> Any:
+    def visualize(self, *args: Any, **kwargs: Any) -> Any:
         """Render the task graph with graphviz.
 
         See :py:func:`dask.visualize` for supported keyword arguments.
@@ -360,8 +370,8 @@ class Histogram(bh.Histogram, family=dask_histogram):
             counts = da.from_array(counts)
             edges = [da.from_array(ea) for ea in edges]  # type: ignore
             if dd:
-                return counts, edges
-            return tuple([counts, *edges])
+                return counts, edges  # type: ignore
+            return tuple([counts, *edges])  # type: ignore
 
 
 def histogramdd(
@@ -375,7 +385,7 @@ def histogramdd(
     histogram: Any | None = None,
     storage: storage.Storage = storage.Double(),
     threads: int | None = None,
-) -> (Histogram | tuple[da.Array, ...] | tuple[da.Array, tuple[da.Array, ...]]):
+) -> Any:
     """Histogram Dask data in multiple dimensions.
 
     Parameters
@@ -565,7 +575,7 @@ def histogramdd(
 
     # Create the axes based on the bins and range values.
     axes = []
-    for _, (b, r) in enumerate(zip(bins, range)):  # type: ignore
+    for _, (b, r) in enumerate(zip(bins, range)):
         if r is None:
             axes.append(axis.Variable(b))  # type: ignore
         else:
@@ -591,7 +601,7 @@ def histogram2d(
     histogram: Any | None = None,
     storage: storage.Storage = storage.Double(),
     threads: int | None = None,
-) -> Histogram | tuple[da.Array, ...]:
+) -> Any:
     """Histogram Dask data in two dimensions.
 
     Parameters
@@ -696,7 +706,7 @@ def histogram2d(
     )
 
     if histogram != Histogram:
-        return hist.to_dask_array(flow=False, dd=False)  # type: ignore
+        return hist.to_dask_array(flow=False, dd=False)
     return hist
 
 
@@ -711,7 +721,7 @@ def histogram(
     histogram: Any | None = None,
     storage: storage.Storage = storage.Double(),
     threads: int | None = None,
-) -> Histogram | tuple[da.Array, ...]:
+) -> Any:
     """Histogram Dask data in one dimension.
 
     Parameters
@@ -793,5 +803,5 @@ def histogram(
     )
 
     if histogram != Histogram:
-        return hist.to_dask_array(flow=False, dd=False)  # type: ignore
+        return hist.to_dask_array(flow=False, dd=False)
     return hist
