@@ -3,23 +3,28 @@
 from __future__ import annotations
 
 import operator
-from typing import TYPE_CHECKING, Any, Callable, Mapping, Sequence
+from typing import TYPE_CHECKING, Any, Callable, Hashable, Mapping, Sequence
 
 import boost_histogram as bh
-import dask.array as da
 import numpy as np
-from dask.bag.core import empty_safe_aggregate, partition_all
-from dask.base import DaskMethodsMixin, is_dask_collection, tokenize
+from dask.array.core import Array as DaskArray
+from dask.array.core import asarray
+from dask.bag.core import empty_safe_aggregate
+from dask.base import DaskMethodsMixin, dont_optimize, is_dask_collection, tokenize
+from dask.blockwise import fuse_roots, optimize_blockwise
+from dask.context import globalmethod
+from dask.core import flatten
 from dask.dataframe.core import partitionwise_graph as partitionwise
 from dask.delayed import Delayed
 from dask.highlevelgraph import HighLevelGraph
 from dask.threaded import get as tget
 from dask.utils import is_dataframe_like, key_split
+from tlz import partition_all
 
 if TYPE_CHECKING:
     from numpy.typing import NDArray
 
-    from .typing import DaskCollection
+    from dask_histogram.typing import DaskCollection
 
 __all__ = (
     "AggHistogram",
@@ -29,7 +34,7 @@ __all__ = (
 )
 
 
-def clone(histref: bh.Histogram = None) -> bh.Histogram:
+def clone(histref: bh.Histogram | None = None) -> bh.Histogram:
     """Create a Histogram object based on another.
 
     The axes and storage of the `histref` will be used to create a new
@@ -54,7 +59,7 @@ def clone(histref: bh.Histogram = None) -> bh.Histogram:
 def _blocked_sa(
     data: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; unweighted; no sample."""
     if data.ndim == 1:
@@ -69,7 +74,7 @@ def _blocked_sa_s(
     data: Any,
     sample: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; unweighted; with sample."""
     if data.ndim == 1:
@@ -84,7 +89,7 @@ def _blocked_sa_w(
     data: Any,
     weights: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; weighted; no sample."""
     if data.ndim == 1:
@@ -100,7 +105,7 @@ def _blocked_sa_w_s(
     weights: Any,
     sample: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; weighted; with sample."""
     if data.ndim == 1:
@@ -113,7 +118,7 @@ def _blocked_sa_w_s(
 
 def _blocked_ma(
     *data: Any,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; multiargument; unweighted; no sample."""
     return clone(histref).fill(*data)
@@ -121,7 +126,7 @@ def _blocked_ma(
 
 def _blocked_ma_s(
     *data: Any,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; multiargument; unweighted; with sample."""
     sample = data[-1]
@@ -131,7 +136,7 @@ def _blocked_ma_s(
 
 def _blocked_ma_w(
     *data: Any,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; multiargument; weighted; no sample."""
     weights = data[-1]
@@ -141,7 +146,7 @@ def _blocked_ma_w(
 
 def _blocked_ma_w_s(
     *data: Any,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; multiargument; weighted; with sample."""
     weights = data[-2]
@@ -153,7 +158,7 @@ def _blocked_ma_w_s(
 def _blocked_df(
     data: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     return clone(histref).fill(*(data[c] for c in data.columns), weight=None)
 
@@ -162,7 +167,7 @@ def _blocked_df_s(
     data: Any,
     sample: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     return clone(histref).fill(*(data[c] for c in data.columns), sample=sample)
 
@@ -171,7 +176,7 @@ def _blocked_df_w(
     data: Any,
     weights: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; weighted; no sample."""
     return clone(histref).fill(*(data[c] for c in data.columns), weight=weights)
@@ -182,7 +187,7 @@ def _blocked_df_w_s(
     weights: Any,
     sample: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
     """Blocked calculation; single argument; weighted; with sample."""
     return clone(histref).fill(
@@ -190,6 +195,32 @@ def _blocked_df_w_s(
     )
 
 
+def _blocked_dak(data: Any, *, histref: bh.Histogram | None = None) -> bh.Histogram:
+    return clone(histref).fill(data)
+
+
+def _blocked_dak_ma(*data: Any, histref: bh.Histogram | None = None) -> bh.Histogram:
+    return clone(histref).fill(*data)
+
+
+def optimize(
+    dsk: Mapping,
+    keys: Hashable | list[Hashable] | set[Hashable],
+    **kwargs: Any,
+) -> Mapping:
+    if not isinstance(keys, (list, set)):
+        keys = [keys]
+    keys = list(flatten(keys))
+
+    if not isinstance(dsk, HighLevelGraph):
+        dsk = HighLevelGraph.from_collections(id(dsk), dsk, dependencies=())
+
+    dsk = optimize_blockwise(dsk, keys=keys)
+    dsk = fuse_roots(dsk, keys=keys)  # type: ignore
+    dsk = dsk.cull(set(keys))  # type: ignore
+    return dsk
+
+
 class AggHistogram(DaskMethodsMixin):
     """Aggregated Histogram collection.
 
@@ -212,7 +243,7 @@ class AggHistogram(DaskMethodsMixin):
     ) -> None:
         self._dask: HighLevelGraph = dsk
         self._name: str = name
-        self._histref: bh.Histogram = histref
+        self._meta: bh.Histogram = histref
 
         # NOTE: Layer only used by `Item.from_delayed`, to handle
         # Delayed objects created by other collections. e.g.:
@@ -244,6 +275,12 @@ class AggHistogram(DaskMethodsMixin):
     def __dask_postpersist__(self) -> Any:
         return self._rebuild, ()
 
+    __dask_optimize__ = globalmethod(
+        optimize, key="histogram_optimize", falsey=dont_optimize
+    )
+
+    __dask_scheduler__ = staticmethod(tget)
+
     def _rebuild(
         self,
         dsk: HighLevelGraph,
@@ -253,7 +290,7 @@ class AggHistogram(DaskMethodsMixin):
         name = self._name
         if rename:
             name = rename.get(name, name)
-        return type(self)(dsk, name, self.meta)
+        return type(self)(dsk, name, self.histref)
 
     @property
     def name(self) -> str:
@@ -266,7 +303,7 @@ class AggHistogram(DaskMethodsMixin):
     @property
     def histref(self) -> bh.Histogram:
         """Empty reference boost-histogram object."""
-        return self._histref
+        return self._meta
 
     @property
     def _storage_type(self) -> type[bh.storage.Storage]:
@@ -298,21 +335,13 @@ class AggHistogram(DaskMethodsMixin):
         )
 
     __repr__ = __str__
-    __dask_scheduler__ = staticmethod(tget)
 
-    @property
-    def _args(self) -> tuple[HighLevelGraph, str, bh.Histogram]:
-        return (self.dask, self.name, self.histref)
-
-    def __getstate__(self) -> tuple[HighLevelGraph, str, bh.Histogram]:
-        return self._args
-
-    def __setstate__(self, state: tuple[HighLevelGraph, str, bh.Histogram]) -> None:
-        self._dask, self._name, self._histref = state
+    def __reduce__(self):
+        return (AggHistogram, (self._dask, self._name, self._meta))
 
     def to_dask_array(
         self, flow: bool = False, dd: bool = False
-    ) -> tuple[da.Array, ...] | tuple[da.Array, tuple[da.Array, ...]]:
+    ) -> tuple[DaskArray, ...] | tuple[DaskArray, list[DaskArray]]:
         """Convert histogram object to dask.array form.
 
         Parameters
@@ -341,23 +370,29 @@ class AggHistogram(DaskMethodsMixin):
         """
         return self.compute()
 
-    def to_delayed(self) -> Delayed:
-        dsk = self.__dask_graph__()
-        return Delayed(self.name, dsk, layer=self._layer)
+    def to_delayed(self, optimize_graph: bool = True) -> Delayed:
+        keys = self.__dask_keys__()
+        graph = self.__dask_graph__()
+        layer = self.__dask_layers__()[0]
+        if optimize_graph:
+            graph = self.__dask_optimize__(graph, keys)
+            layer = f"delayed-{self.name}"
+            graph = HighLevelGraph.from_collections(layer, graph, dependencies=())
+        return Delayed(keys[0], graph, layer=layer)
 
     def values(self, flow: bool = False) -> NDArray[Any]:
-        return self.to_boost().values()
+        return self.to_boost().values(flow=flow)
 
     def variances(self, flow: bool = False) -> NDArray[Any] | None:
-        return self.to_boost().variances()
+        return self.to_boost().variances(flow=flow)
 
     def counts(self, flow: bool = False) -> NDArray[Any]:
-        return self.to_boost().counts()
+        return self.to_boost().counts(flow=flow)
 
     def __array__(self) -> NDArray[Any]:
         return self.compute().__array__()
 
-    def __iadd__(self, other) -> AggHistogram:
+    def __iadd__(self, other: Any) -> AggHistogram:
         return _iadd(self, other)
 
     def __add__(self, other: Any) -> AggHistogram:
@@ -420,9 +455,9 @@ class PartitionedHistogram(DaskMethodsMixin):
         self, dsk: HighLevelGraph, name: str, npartitions: int, histref: bh.Histogram
     ) -> None:
         self._dask: HighLevelGraph = dsk
-        self._name = name
+        self._name: str = name
         self._npartitions: int = npartitions
-        self._histref: bh.Histogram = histref
+        self._meta: bh.Histogram = histref
 
     @property
     def name(self) -> str:
@@ -457,6 +492,12 @@ class PartitionedHistogram(DaskMethodsMixin):
             name = rename.get(name, name)
         return type(self)(dsk, name, self.npartitions, self.histref)
 
+    __dask_optimize__ = globalmethod(
+        optimize, key="histogram_optimize", falsey=dont_optimize
+    )
+
+    __dask_scheduler__ = staticmethod(tget)
+
     def __str__(self) -> str:
         return "dask_histogram.PartitionedHistogram,<%s, npartitions=%d>" % (
             key_split(self.name),
@@ -464,31 +505,42 @@ class PartitionedHistogram(DaskMethodsMixin):
         )
 
     __repr__ = __str__
-    __dask_scheduler__ = staticmethod(tget)
 
-    @property
-    def _args(self) -> tuple[HighLevelGraph, str, int, bh.Histogram]:
-        return (self.dask, self.name, self.npartititions, self.histref)
-
-    def __getstate__(self) -> tuple[HighLevelGraph, str, int, bh.Histogram]:
-        return self._args
-
-    def __setstate__(
-        self, state: tuple[HighLevelGraph, str, int, bh.Histogram]
-    ) -> None:
-        self._dask, self._name, self._npartitions, self._histref = state
+    def __reduce__(self):
+        return (
+            PartitionedHistogram,
+            (
+                self._dask,
+                self._name,
+                self._npartitions,
+                self._meta,
+            ),
+        )
 
     @property
     def histref(self) -> bh.Histogram:
         """boost_histogram.Histogram: reference histogram."""
-        return self._histref
+        return self._meta
 
-    def to_agg(self, split_every: int = None) -> AggHistogram:
+    def collapse(self, split_every: int | None = None) -> AggHistogram:
         """Translate into a reduced aggregated histogram."""
         return _reduction(self, split_every=split_every)
 
+    def to_delayed(self, optimize_graph: bool = True) -> list[Delayed]:
+        keys = self.__dask_keys__()
+        graph = self.__dask_graph__()
+        layer = self.__dask_layers__()[0]
+        if optimize_graph:
+            graph = self.__dask_optimize__(graph, keys)
+            layer = f"delayed-{self.name}"
+            graph = HighLevelGraph.from_collections(layer, graph, dependencies=())
+        return [Delayed(k, graph, layer=layer) for k in keys]
 
-def _reduction(ph: PartitionedHistogram, split_every: int = None) -> AggHistogram:
+
+def _reduction(
+    ph: PartitionedHistogram,
+    split_every: int | None = None,
+) -> AggHistogram:
     if split_every is None:
         split_every = 4
     if split_every is False:
@@ -564,11 +616,28 @@ def _partitioned_histogram(
     weights: DaskCollection | None = None,
     sample: DaskCollection | None = None,
     split_every: int | None = None,
-) -> AggHistogram:
-    name = f"hist-on-block-{tokenize(data, histref, weights, sample)}"
+) -> PartitionedHistogram:
+    name = f"hist-on-block-{tokenize(data, histref, weights, sample, split_every)}"
     data_is_df = is_dataframe_like(data[0])
+    data_is_dak = is_awkward_like(data[0])
     _weight_sample_check(*data, weights=weights)
-    if len(data) == 1 and not data_is_df:
+
+    # Single awkward array object.
+    if len(data) == 1 and data_is_dak:
+        from dask_awkward.core import partitionwise_layer as dak_pwl
+
+        x = data[0]
+        if weights is not None and sample is not None:
+            raise NotImplementedError()
+        elif weights is not None and sample is None:
+            raise NotImplementedError()
+        elif weights is None and sample is not None:
+            raise NotImplementedError()
+        else:
+            g = dak_pwl(_blocked_dak, name, x, histref=histref)
+
+    # Single object, not a dataframe
+    elif len(data) == 1 and not data_is_df:
         x = data[0]
         if weights is not None and sample is not None:
             g = partitionwise(
@@ -580,6 +649,8 @@ def _partitioned_histogram(
             g = partitionwise(_blocked_sa_s, name, x, sample, histref=histref)
         else:
             g = partitionwise(_blocked_sa, name, x, histref=histref)
+
+    # Single object, is a dataframe
     elif len(data) == 1 and data_is_df:
         x = data[0]
         if weights is not None and sample is not None:
@@ -592,8 +663,20 @@ def _partitioned_histogram(
             g = partitionwise(_blocked_df_s, name, x, sample, histref=histref)
         else:
             g = partitionwise(_blocked_df, name, x, histref=histref)
+
+    # Multiple objects
     else:
-        if weights is not None and sample is not None:
+
+        # Awkward array collection detected as first argument
+        if data_is_dak:
+            from dask_awkward.core import partitionwise_layer as dak_pwl
+
+            if weights is None and sample is None:
+                g = dak_pwl(_blocked_dak_ma, name, *data, histref=histref)
+            else:
+                raise NotImplementedError()
+        # Not an awkward array collection
+        elif weights is not None and sample is not None:
             g = partitionwise(
                 _blocked_ma_w_s, name, *data, weights, sample, histref=histref
             )
@@ -609,28 +692,11 @@ def _partitioned_histogram(
     return PartitionedHistogram(hlg, name, data[0].npartitions, histref=histref)
 
 
-def _reduced_histogram(
-    *data: DaskCollection,
-    histref: bh.Histogram,
-    weights: DaskCollection | None = None,
-    sample: DaskCollection | None = None,
-    split_every: int | None = None,
-) -> AggHistogram:
-    ph = _partitioned_histogram(
-        *data,
-        histref=histref,
-        weights=weights,
-        sample=sample,
-        split_every=split_every,
-    )
-    return ph.to_agg(split_every=split_every)
-
-
 def to_dask_array(
     agghist: AggHistogram,
     flow: bool = False,
     dd: bool = False,
-) -> tuple[DaskCollection, list[DaskCollection]] | tuple[DaskCollection, ...]:
+) -> tuple[DaskArray, ...] | tuple[DaskArray, list[DaskArray]]:
     """Convert `agghist` to a `dask.array` return style.
 
     Parameters
@@ -668,22 +734,26 @@ def to_dask_array(
         bh.storage.AtomicInt64,
     )
     dt = int if int_storage else float
-    c = da.Array(graph, name=name, shape=shape, chunks=shape, dtype=dt)
+    c = DaskArray(graph, name=name, shape=shape, chunks=shape, dtype=dt)
     axes = agghist.histref.axes
 
     if flow:
         edges = [
-            da.asarray(np.concatenate([[-np.inf], ax.edges, [np.inf]])) for ax in axes
+            asarray(np.concatenate([[-np.inf], ax.edges, [np.inf]])) for ax in axes
         ]
     else:
-        edges = [da.asarray(ax.edges) for ax in axes]
+        edges = [asarray(ax.edges) for ax in axes]
     if dd:
         return c, edges
     return (c, *tuple(edges))
 
 
 class BinaryOpAgg:
-    def __init__(self, func: Callable[[Any, Any], Any], name: str = None) -> None:
+    def __init__(
+        self,
+        func: Callable[[Any, Any], Any],
+        name: str | None = None,
+    ) -> None:
         self.func = func
         self.__name__ = func.__name__ if name is None else name
 
@@ -694,12 +764,12 @@ class BinaryOpAgg:
             deps.append(a)
             k1 = a.name
         else:
-            k1 = a
+            k1 = a  # type: ignore
         if is_dask_collection(b):
             deps.append(b)
             k2 = b.name
         else:
-            k2 = b
+            k2 = b  # type: ignore
         k1 = a.__dask_tokenize__() if is_dask_collection(a) else a  # type: ignore
         k2 = b.__dask_tokenize__() if is_dask_collection(b) else b  # type: ignore
         llg = {name: (self.func, k1, k2)}
@@ -726,7 +796,7 @@ def factory(
     sample: DaskCollection | None = None,
     split_every: int | None = None,
     keep_partitioned: bool = False,
-) -> AggHistogram | PartitionedHistogram:
+) -> AggHistogram:
     """Daskified Histogram collection factory function.
 
     Given some data represented by Dask collections and the
@@ -823,6 +893,30 @@ def factory(
     dask.array<array, shape=(11,), dtype=float64, chunksize=(11,), chunktype=numpy.ndarray>
 
     """
+    if keep_partitioned:
+        raise ValueError(
+            "keep_partitioned=True is no longer supported; "
+            "use dask_histogram.partitioned_factory."
+        )
+    ph = partitioned_factory(
+        *data,
+        histref=histref,
+        axes=axes,
+        storage=storage,
+        weights=weights,
+        sample=sample,
+    )
+    return ph.collapse(split_every=split_every)
+
+
+def partitioned_factory(
+    *data: DaskCollection,
+    histref: bh.Histogram | None = None,
+    axes: Sequence[bh.axis.Axis] | None = None,
+    storage: bh.storage.Storage | None = None,
+    weights: DaskCollection | None = None,
+    sample: DaskCollection | None = None,
+) -> PartitionedHistogram:
     if histref is None and axes is None:
         raise ValueError("Either histref or axes must be defined.")
     if histref is not None and storage is not None:
@@ -831,11 +925,26 @@ def factory(
         if storage is None:
             storage = bh.storage.Double()
         histref = bh.Histogram(*axes, storage=storage)  # type: ignore
-    f = _partitioned_histogram if keep_partitioned else _reduced_histogram
-    return f(
-        *data,
-        histref=histref,
-        weights=weights,
-        sample=sample,
-        split_every=split_every,
+
+    return _partitioned_histogram(
+        *data, histref=histref, weights=weights, sample=sample
+    )
+
+
+def is_awkward_like(x: Any) -> bool:
+    """Check if an object is Awkward collection like.
+
+    Parameters
+    ----------
+    x : Any
+        The object of interest.
+
+    Returns
+    -------
+    bool
+        ``True`` if `x` is an Awkward Dask collection.
+
+    """
+    return (
+        hasattr(x, "__dask_graph__") and hasattr(x, "layout") and hasattr(x, "fields")
     )
